@@ -4,7 +4,7 @@ package UR::Namespace::Command::Update::ClassesFromDb;
 use strict;
 use warnings;
 use UR;
-our $VERSION = "0.40"; # UR $VERSION;
+our $VERSION = "0.43"; # UR $VERSION;
 use Text::Diff;
 
 UR::Object::Type->define(
@@ -104,7 +104,8 @@ sub execute {
         if ($specified_class_name_arrayref) {
             $ds_table_list = [
                 map { [$_->data_source, $_->table_name] }
-                map { $_->__meta__ } 
+                grep { $_->data_source }
+                map { $_->__meta__ }
                 @$specified_class_name_arrayref
             ];        
         }
@@ -584,49 +585,45 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
 
     my $namespace = $self->namespace_name;
 
-=cut
-
-    $self->status_message("Using filesystem classes for namespace \"$namespace\" (this may be slow)");
-    my @material_classes = $namespace->get_material_classes;
-
-
-    $self->status_message("Verifying class/table relationships...");
-    my %table_ids_used;
-    for my $class (sort { $a->class_name cmp $b->class_name } @material_classes) {
-        my $table_name  = $class->table_name;
-        next unless $table_name;
-
-        my $class_name  = $class->class_name;
-
-        if (my $prev_class_name = $table_ids_used{$table_name}) {
-            $self->error_message(
-                sprintf(
-                    "C %-40s uses table %-32s, but so does %-40s" . "\n",
-                    $class_name, $table_name, $prev_class_name
-                )
-            );
-            return;
-        }
-
-        my $data_source = $class->data_source;
-
-        my $table = UR::DataSource::RDBMS::Table->get(data_source => $data_source, table_name => $table_name)
-                    ||
-                    UR::DataSource::RDBMS::Table::Ghost->get(data_source => $data_source, table_name => $table_name);
-
-        unless ($table) {
-            $self->error_message(
-                sprintf(
-                    "C %-32s %-32s is referenced by class %-40s but cannot be found!?" . "\n",
-                    $data_source, $table_name, $class_name
-                )
-            );
-            return;
-        }
-        $table_ids_used{$table_name} = $class;
-    }
-
-=cut
+#    $self->status_message("Using filesystem classes for namespace \"$namespace\" (this may be slow)");
+#    my @material_classes = $namespace->get_material_classes;
+#
+#
+#    $self->status_message("Verifying class/table relationships...");
+#    my %table_ids_used;
+#    for my $class (sort { $a->class_name cmp $b->class_name } @material_classes) {
+#        my $table_name  = $class->table_name;
+#        next unless $table_name;
+#
+#        my $class_name  = $class->class_name;
+#
+#        if (my $prev_class_name = $table_ids_used{$table_name}) {
+#            $self->error_message(
+#                sprintf(
+#                    "C %-40s uses table %-32s, but so does %-40s" . "\n",
+#                    $class_name, $table_name, $prev_class_name
+#                )
+#            );
+#            return;
+#        }
+#
+#        my $data_source = $class->data_source;
+#
+#        my $table = UR::DataSource::RDBMS::Table->get(data_source => $data_source, table_name => $table_name)
+#                    ||
+#                    UR::DataSource::RDBMS::Table::Ghost->get(data_source => $data_source, table_name => $table_name);
+#
+#        unless ($table) {
+#            $self->error_message(
+#                sprintf(
+#                    "C %-32s %-32s is referenced by class %-40s but cannot be found!?" . "\n",
+#                    $data_source, $table_name, $class_name
+#                )
+#            );
+#            return;
+#        }
+#        $table_ids_used{$table_name} = $class;
+#    }
 
     $self->status_message("Updating classes...");
 
@@ -853,6 +850,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         else {
             # create
             my $data_source = $table->data_source;
+            my $data_source_id = (ref $data_source ? $data_source->id : $data_source);
             my $class_name = $data_source->resolve_class_name_for_table_name($table_name,$table->table_type);
             unless ($class_name) {
                 Carp::confess(
@@ -865,12 +863,13 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             # new one actually has a table, then this is just another schema change and
             # not an error.  Set the table_name attribute and go on...
             my $class = UR::Object::Type->get(class_name => $class_name);
-            my $prev_table_name = $class->table_name if ($class);
+            my $prev_table_name = ($class ? $class->table_name : undef);
+            my $prev_data_source_id = ($class ? $class->data_source_id : undef);
             if ($class && $prev_table_name) {
 
                 Carp::confess(
-                    "Class $class_name already exists for table '$prev_table_name'."
-                    . "  Cannot generate class for $table_name."
+                    "Class $class_name already exists for table '$prev_table_name' in $prev_data_source_id."
+                    . "  Cannot generate class for $table_name in $data_source_id."
                 );
             }
 
@@ -921,18 +920,12 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
     $self->status_message($_) foreach @saved_removed_column_messages;
 
     # COLUMN
-    my @column_property_translations = (
-        ['data_type'    => 'data_type'],
-        ['data_length'  => 'data_length'],
-        ['nullable'     => 'is_optional', sub { (defined($_[0]) and ($_[0] eq "Y")) ? 1 : 0 } ],
-        ['remarks'      => 'doc'],
-    );
     
     for my $column (sort $sorter @{ $dd_changes_by_class{'UR::DataSource::RDBMS::TableColumn'} }) {
         my $table = $column->get_table;
         my $column_name = $column->column_name;
         my $data_source = $table->data_source;
-        my($ur_data_type,$default_length) = @{ $data_source->ur_data_type_for_data_source_data_type($column->data_type) };
+        my($ur_data_type, $default_length) = @{ $data_source->ur_data_type_for_data_source_data_type($column->data_type) };
         my $ur_data_length = defined($column->data_length) ? $column->data_length : $default_length;
 
         #my $class = UR::Object::Type->get(
@@ -960,6 +953,21 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
 
         # We care less whether the column is new/updated, than whether there is property metadata for it.
         if ($property) {
+            my @column_property_translations = (
+                # [ column_name, property_name, conversion_sub(column_obj, value) ]
+                ['data_length'  => 'data_length',
+                    # lengths for these data types are based on the number of bytes used internally in the
+                    # database.  The UR-based objects will store the text version, which will always be longer,
+                    # making $obj->__errors__() complain about the length being out of bounds
+                    sub { my ($c, $av) = @_;  defined($av) ? $av : ($c->is_time_data ? undef : $ur_data_length) } ],
+                ['data_type'    => 'data_type',
+                    sub { my ($c, $av) = @_;  defined($ur_data_type) ? $ur_data_type : $av } ],
+                ['nullable'     => 'is_optional',
+                    sub { my ($c, $av) = @_; (defined($av) and ($av eq "Y")) ? 1 : 0 } ],
+                ['remarks'      => 'doc',
+                    # Ideally this would only use DB value ($av) if the last_ddl_time was newer.
+                    sub { my ($c, $av) = @_; defined($av) ? $av : $property->doc } ],
+            );
             # update
             for my $translation (@column_property_translations) {
                 my ($column_attr, $property_attr, $conversion_sub) = @$translation;
@@ -968,22 +976,14 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                 no warnings;
                 if (UR::Context->_get_committed_property_value($column,$column_attr) ne $column->$column_attr) {
                     if ($conversion_sub) {
-                        $property->$property_attr($conversion_sub->($column->$column_attr));
+                        $property->$property_attr($conversion_sub->($column, $column->$column_attr));
                     }
                     else {
                         $property->$property_attr($column->$column_attr);
                     }
                 }
             }
-            $property->data_type($ur_data_type) if (! defined $property->data_type);
-            # lengths for these data types are based on the number of bytes used internally in the
-            # database.  The UR-based objects will store the text version, which will always be longer,
-            # making $obj->__errors__() complain about the length being out of bounds
-            $property->data_length($column->is_time_data ? undef : $ur_data_length) if (! defined $property->data_length);
 
-            $property->is_optional($column->nullable eq "Y" ? 1 : 0);
-            $property->doc($column->remarks);
-            
             if ($property->__changes__) {
                 no warnings;
                 $self->status_message(
@@ -1146,7 +1146,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             eval { $class->remove_unique_constraint($uc_name) };
             if ($@ =~ m/There is no constraint named/) {
                 next;  # it's OK if there's no UR metadata for this constraint yet
-            } else {
+            } elsif ($@) {
                 die $@;
             }
 
